@@ -12,6 +12,9 @@ from pyrogram.errors.exceptions.bad_request_400 import AccessTokenExpired
 
 from d4rk.Logs import setup_logger
 
+logs_sent = False
+logs_lock = asyncio.Lock()
+
 logger = setup_logger(__name__)
 
 class BotManager(Client):
@@ -28,6 +31,7 @@ class BotManager(Client):
 
 
     def create_client(self,app_name,token):
+        self.app_name = app_name
         super().__init__(
             name=app_name,
             api_id=self.api_id,
@@ -41,20 +45,17 @@ class BotManager(Client):
         logger.info(f"FloodWait: Sleeping for {wait_time} seconds.")
 
         await asyncio.sleep(wait_time)
-        try:await super().start()
-        except AccessTokenExpired:pass
+        try:self.powerup(self.app_name)
+        except:pass
         
     def _safe_async(self, coro_func):
         if self._loop:asyncio.run_coroutine_threadsafe(coro_func(), self._loop)
         else:logger.error("Event loop is not set for _safe_async")
 
-    
-
     async def powerup(self,appname):
         if hasattr(self, "db"):
             self.font = self.db.Settings.get(key="font",datatype=str)
             self.sudo_users = self.db.Settings.get(key="sudo_users",datatype=list,default=[])
-
             if not self.font:
                 logger.info("Font not set, defaulting to font 1")
                 self.db.Settings.set("font", "1")
@@ -84,6 +85,8 @@ class BotManager(Client):
             except FloodWait as e:
                 logger.error(f"FloodWait: {e.value} seconds")
                 await self.handle_flood_wait(e.value)
+                break
+
             except AccessTokenExpired:
                 logger.error(f"Access token expired (attempt {attempt + 1})")
             except Exception as e:
@@ -98,22 +101,35 @@ class BotManager(Client):
         await self.setup_webserver()
 
     async def powerdown(self, *args):
+        global logs_sent, logs_lock
         logger.info("Initiating APP to stop...")
         if self._rename:await super().set_bot_info(lang_code='en',name=self.app_name + " (Offline)")
         self.stop_scheduler()
         today = self.TZ_now.strftime("%Y-%m-%d")
         if hasattr(self, '_web_runner') and self._web_runner:
             await self.web_server.cleanup()
-        if self._is_connected:
-            logger.info("Stopping bot client...")
+        
+        logger.info("Stopping bot client...")
+        if self._is_connected and self.LOGS:
             try:
-                if self.LOGS:
-                    await self.send_message(chat_id=self.LOGS, text=f"{self._bot_info.mention} stopping...")
-                    await self.send_document(chat_id=self.LOGS, document=f"logs/log-{today}.txt")
-                    await self.send_message(chat_id=self.LOGS, text=f"{self._bot_info.mention} stopped successfully!")
+                await self.send_message(chat_id=self.LOGS, text=f"{self._bot_info.mention} stopping...")
             except Exception as e:
-                logger.error(f"Error sending stop notification: {e}")
-            logger.info(f"{self._bot_info.first_name} - @{self._bot_info.username} Stoped")
+                logger.error(f"Error sending stop message for {self._bot_info.first_name}: {e}")
+
+        async with logs_lock:
+            if not logs_sent and self._is_connected:
+                logs_sent = True
+                try:
+                    await self.send_document(chat_id=self.LOGS, document=f"logs/log-{today}.txt")
+                    logger.info("Log document sent successfully")
+                except Exception as e:
+                    logger.error(f"Error sending log document: {e}")
+
+        if self._is_connected and self.LOGS:
+            try:
+                await self.send_message(chat_id=self.LOGS, text=f"{self._bot_info.mention} stopped successfully!")
+            except Exception as e:
+                logger.error(f"Error sending stop confirmation for {self._bot_info.first_name}: {e}")
             await super().stop()
             sys.exit(0)
 
@@ -153,8 +169,10 @@ class BotManager(Client):
             except Exception as e:logger.error(f"Failed to send restart notification: {e}")
             try:await self.edit_message_text(chat_id=chat_id,message_id=Message_id, text="Bot restarted successfully!")          
             except:
-                await self.send_message(chat_id=chat_id, text="Bot restarted successfully!",reply_to_message_id=Message_id-1,)
-                await self.delete_messages(chat_id=chat_id,message_ids=Message_id)
+                try:
+                    await self.send_message(chat_id=chat_id, text="Bot restarted successfully!",reply_to_message_id=Message_id-1,)
+                    await self.delete_messages(chat_id=chat_id,message_ids=Message_id)
+                except:pass
             os.remove('restart.txt')
         else:
             try:await self.send_message(chat_id=self.LOGS, text=f"{self._bot_info.mention} started successfully!")
