@@ -5,4 +5,4 @@ from ._delete import delete
 from ._ractions import Reacts
 from ._fonts import FontMessageMixin , get_font , web_app_data
 from ._movie_parser import parser
-from ._round import round_robin
\ No newline at end of file
+from ._round import round_robin , round
\ No newline at end of file
